import { NetworkelementAddListRequest, NetworkelementSetMne, useNetworkElementServiceAddListMutation } from "@api/api";
import { useAppDispatch, useAppSelector } from "@hooks";
import { fetchPnds, fetchUser } from "@shared/routine/user.routine";
import { useState } from "react";
import { SubmitHandler, useForm } from "react-hook-form";
import { useTranslation } from "react-i18next";
import { toast } from "react-toastify";

interface FormData {
    address: string;
    mneName: string;
    pluginId: string;
    transportOption: {
        address: string;
        username: string;
        password: string;
        tls: boolean;
        gnmiTransportOption: Object;
    };
    gnmiSubscribePaths: string[];
}

type ModalViewModelType = {
    hide: () => void
}

export const useModalViewModel = ({ hide }: ModalViewModelType) => {
    const dispatch = useAppDispatch()
    const [addNetworkElement] = useNetworkElementServiceAddListMutation();
    const [loading, setLoading] = useState<boolean>(false)
    const { t } = useTranslation('common')

    const {
        register,
        handleSubmit,
        setError,
        clearErrors,
        formState: { errors },
        reset: resetModal
    } = useForm<FormData>();

    const { user } = useAppSelector(state => state.user);

    const reset = () => { resetModal(); hide(); }
    const success = () => {
        toast.success(t('device.add_device.success'));
        reset();
        dispatch(fetchPnds())
        dispatch(fetchUser())
    }


    const onSubmit: SubmitHandler<FormData> = async (data) => {
        clearErrors()
        setLoading(true)
        const mne: NetworkelementSetMne = {
            ...data,
            gnmiSubscribePaths: [],
            transportOption: {
                ...data.transportOption,
                gnmiTransportOption: {},
            }
        }

        if (!user?.id || !user?.roles) {
            toast.error("global.error.missing_user")
            return
        }

        const request: NetworkelementAddListRequest = {
            timestamp: Date.now().toString(),
            mne: [mne],
            pid: Object.keys(user.roles)[0]
        };

        try {
            const response = await addNetworkElement({ networkelementAddListRequest: request });
            setLoading(false)

            if (response.error) {
                const error = response.error as any
                setError('root', error.data)
                return
            }

            success()
        } catch (error) {
            setError('root', error?.data?.message || undefined)
        }
    }

    return {
        onSubmit,
        handleSubmit: handleSubmit(onSubmit),
        register,
        errors,
        loading
    }
}