// login.view.tsx
import { BasicProp } from "@shared/types/interfaces.type";
import React from "react";
import {
  Alert,
  Button,
  Col,
  Container,
  Form,
  Image,
  Row,
  Spinner,
} from "react-bootstrap";
import { useForm } from "react-hook-form";
import { useTranslation } from "react-i18next";
import useLoginViewModel from "../viewmodel/login.viewmodel";
import "./login.scss";
import { LoginFormInputs } from "./login.types";
import logo from "/public/logo.svg";

export const LoginView: React.FC<BasicProp> = () => {
  const { t } = useTranslation("common");
  const { handleLogin, getErrorMessage, loginLoading } = useLoginViewModel();

  const {
    register,
    handleSubmit,
    formState: { errors },
  } = useForm<LoginFormInputs>({
    mode: "onSubmit",
    defaultValues: {
      username: "",
      password: "",
    },
  });

  const invalidForm = (
    <Alert variant="warning">{t("login.form.invalid")}</Alert>
  );
  const invalidCredentials = (
    <Alert variant="danger">{t("login.form.failed")}</Alert>
  );

  return (
    <Container className="vh-100 d-flex flex-column justify-content-center login-container">
      <Row className="align-items-center">
        <Image src={logo} alt="logo" height={150} />
      </Row>
      <Row className="mt-2 justify-content-center">
        <Col md={6} sm={10} className="c-box p-4 bg-white">
          <h1 className="text-center h2">goSDN - Web</h1>

          {getErrorMessage(invalidForm, invalidCredentials)}

          <Form className="mt-4" onSubmit={handleSubmit(handleLogin)}>
            <Form.Group className="mb-3" controlId="loginForm.username">
              <Form.Label>{t("login.form.username.label")}</Form.Label>
              <Form.Control
                type="text"
                isInvalid={!!errors.username}
                {...register("username", {
                  required: t("global.form.empty_field"),
                })}
                autoComplete="on"
                autoFocus={true}
              />
              <Form.Control.Feedback type="invalid">
                {errors.username?.message}
              </Form.Control.Feedback>
            </Form.Group>
            <Form.Group className="mb-3" controlId="loginForm.password">
              <Form.Label>{t("login.form.password.label")}</Form.Label>
              <Form.Control
                type="password"
                isInvalid={!!errors.password}
                {...register("password", {
                  required: t("global.form.empty_field"),
                })}
              />
              <Form.Control.Feedback type="invalid">
                {errors.password?.message}
              </Form.Control.Feedback>
            </Form.Group>
            <Button
              variant="primary"
              type="submit"
              className="w-100 mt-3"
              disabled={loginLoading}
            >
              {t("global.form.submit")}
              {loginLoading && (
                <Spinner animation="border" size="sm" role="status">
                  <span className="visually-hidden">Loading...</span>
                </Spinner>
              )}
            </Button>
          </Form>
        </Col>
      </Row>
    </Container>
  );
};

export default LoginView;
